@@ -2,6 +2,7 @@ import os
 import sys
 from robotframework_ls.constants import NULL
 from robocorp_ls_core.robotframework_log import get_logger
+import threading
 
 log = get_logger(__name__)
 
@@ -228,61 +229,66 @@ class _FolderInfo(object):
         self.recursive = recursive
         self.libspec_canonical_filename_to_info = {}
         self._watch = NULL
+        self._lock = threading.Lock()
 
     def start_watch(self, observer, notifier):
-        if self._watch is NULL:
-            if not os.path.isdir(self.folder_path):
-                if not os.path.exists(self.folder_path):
-                    log.info(
-                        "Trying to track changes in path which does not exist: %s",
-                        self.folder_path,
-                    )
-                else:
-                    log.info(
-                        "Trying to track changes in path which is not a folder: %s",
-                        self.folder_path,
-                    )
-                return
+        with self._lock:
+            if self._watch is NULL:
+                if not os.path.isdir(self.folder_path):
+                    if not os.path.exists(self.folder_path):
+                        log.info(
+                            "Trying to track changes in path which does not exist: %s",
+                            self.folder_path,
+                        )
+                    else:
+                        log.info(
+                            "Trying to track changes in path which is not a folder: %s",
+                            self.folder_path,
+                        )
+                    return
 
-            log.info("Tracking folder for changes: %s", self.folder_path)
-            from robocorp_ls_core.watchdog_wrapper import PathInfo
+                log.info("Tracking folder for changes: %s", self.folder_path)
+                from robocorp_ls_core.watchdog_wrapper import PathInfo
 
-            folder_path = self.folder_path
-            self._watch = observer.notify_on_extensions_change(
-                [PathInfo(folder_path, recursive=self.recursive)],
-                ["libspec"],
-                notifier.on_change,
-                (self._on_change_spec,),
-            )
+                folder_path = self.folder_path
+                self._watch = observer.notify_on_extensions_change(
+                    [PathInfo(folder_path, recursive=self.recursive)],
+                    ["libspec"],
+                    notifier.on_change,
+                    (self._on_change_spec,),
+                )
 
     def _on_change_spec(self, spec_file):
-        spec_file_key = _norm_filename(spec_file)
-        # Just add/remove that specific spec file from the tracked list.
-        libspec_canonical_filename_to_info = (
-            self.libspec_canonical_filename_to_info.copy()
-        )
-        if os.path.exists(spec_file):
-            libspec_canonical_filename_to_info[spec_file_key] = None
-        else:
-            libspec_canonical_filename_to_info.pop(spec_file_key, None)
+        with self._lock:
+            spec_file_key = _norm_filename(spec_file)
+            # Just add/remove that specific spec file from the tracked list.
+            libspec_canonical_filename_to_info = (
+                self.libspec_canonical_filename_to_info.copy()
+            )
+            if os.path.exists(spec_file):
+                libspec_canonical_filename_to_info[spec_file_key] = None
+            else:
+                libspec_canonical_filename_to_info.pop(spec_file_key, None)
 
-        self.libspec_canonical_filename_to_info = libspec_canonical_filename_to_info
+            self.libspec_canonical_filename_to_info = libspec_canonical_filename_to_info
 
     def synchronize(self):
-        try:
-            self.libspec_canonical_filename_to_info = self._collect_libspec_info(
-                [self.folder_path],
-                self.libspec_canonical_filename_to_info,
-                recursive=self.recursive,
-            )
-        except Exception:
-            log.exception("Error when synchronizing: %s", self.folder_path)
+        with self._lock:
+            try:
+                self.libspec_canonical_filename_to_info = self._collect_libspec_info(
+                    [self.folder_path],
+                    self.libspec_canonical_filename_to_info,
+                    recursive=self.recursive,
+                )
+            except Exception:
+                log.exception("Error when synchronizing: %s", self.folder_path)
 
     def dispose(self):
-        watch = self._watch
-        self._watch = NULL
-        watch.stop_tracking()
-        self.libspec_canonical_filename_to_info = {}
+        with self._lock:
+            watch = self._watch
+            self._watch = NULL
+            watch.stop_tracking()
+            self.libspec_canonical_filename_to_info = {}
 
     def _collect_libspec_info(self, folders, old_libspec_filename_to_info, recursive):
         seen_libspec_files = set()
@@ -372,13 +378,7 @@ class LibspecManager(object):
         """
         from robocorp_ls_core import watchdog_wrapper
 
-        from concurrent import futures
-
-        from multiprocessing import cpu_count
-
-        self._thread_pool = futures.ThreadPoolExecutor(
-            max_workers=(cpu_count() * 1.2) + 1
-        )
+        self._main_thread = threading.current_thread()
 
         self._observer = watchdog_wrapper.create_observer()
 
@@ -435,8 +435,18 @@ class LibspecManager(object):
         # Must be set from the outside world when needed.
         self.config = None
 
-        self._synchronize()
+        log.debug("Generating builtin libraries.")
         self._gen_builtin_libraries()
+        log.debug("Synchronizing internal caches.")
+        self._synchronize()
+        log.debug("Finished initializing LibspecManager.")
+
+    def _check_in_main_thread(self):
+        curr_thread = threading.current_thread()
+        if self._main_thread is not curr_thread:
+            raise AssertionError(
+                f"This may only be called at the thread: {self._main_thread}. Current thread: {curr_thread}"
+            )
 
     @property
     def config(self):
@@ -444,6 +454,7 @@ class LibspecManager(object):
 
     @config.setter
     def config(self, config):
+        self._check_in_main_thread()
         from robotframework_ls.impl.robot_lsp_constants import OPTION_ROBOT_PYTHONPATH
 
         self._config = config
@@ -470,6 +481,7 @@ class LibspecManager(object):
         target(spec_file)
 
     def add_workspace_folder(self, folder_uri):
+        self._check_in_main_thread()
         from robocorp_ls_core import uris
 
         if folder_uri not in self._workspace_folder_uri_to_folder_info:
@@ -485,6 +497,7 @@ class LibspecManager(object):
             log.debug("Workspace folder already added: %s", folder_uri)
 
     def remove_workspace_folder(self, folder_uri):
+        self._check_in_main_thread()
         if folder_uri in self._workspace_folder_uri_to_folder_info:
             log.debug("Removed workspace folder: %s", folder_uri)
             cp = self._workspace_folder_uri_to_folder_info.copy()
@@ -495,6 +508,7 @@ class LibspecManager(object):
             log.debug("Workspace folder already removed: %s", folder_uri)
 
     def add_additional_pythonpath_folder(self, folder_path):
+        self._check_in_main_thread()
         if folder_path not in self._additional_pythonpath_folder_to_folder_info:
             log.debug("Added additional pythonpath folder: %s", folder_path)
             cp = self._additional_pythonpath_folder_to_folder_info.copy()
@@ -506,6 +520,7 @@ class LibspecManager(object):
             log.debug("Additional pythonpath folder already added: %s", folder_path)
 
     def remove_additional_pythonpath_folder(self, folder_path):
+        self._check_in_main_thread()
         if folder_path in self._additional_pythonpath_folder_to_folder_info:
             log.debug("Removed additional pythonpath folder: %s", folder_path)
             cp = self._additional_pythonpath_folder_to_folder_info.copy()
@@ -519,9 +534,10 @@ class LibspecManager(object):
         """
         Generates .libspec files for the libraries builtin (if needed).
         """
-        import time
 
         try:
+            import time
+            from concurrent import futures
             from robotframework_ls.impl import robot_constants
             from robocorp_ls_core.system_mutex import timed_acquire_mutex
             from robocorp_ls_core.system_mutex import generate_mutex_name
@@ -529,31 +545,44 @@ class LibspecManager(object):
             initial_time = time.time()
             wait_for = []
 
-            with timed_acquire_mutex(
-                generate_mutex_name(
-                    _norm_filename(self._builtins_libspec_dir), prefix="gen_builtins_"
-                ),
-                timeout=100,
-            ):
-                for libname in robot_constants.STDLIBS:
-                    library_info = self.get_library_info(libname, create=False)
-                    if library_info is None:
-                        wait_for.append(
-                            self._thread_pool.submit(
-                                self._create_libspec, libname, is_builtin=True
-                            )
-                        )
-                for future in wait_for:
-                    future.result()
+            max_workers = min(10, (os.cpu_count() or 1) + 4)
+            thread_pool = futures.ThreadPoolExecutor(max_workers=max_workers)
 
-            if wait_for:
-                log.debug(
-                    "Total time to generate builtins: %.2fs"
-                    % (time.time() - initial_time)
-                )
-                self.synchronize_internal_libspec_folders()
+            try:
+                log.debug("Waiting for mutex to generate builtins.")
+                with timed_acquire_mutex(
+                    generate_mutex_name(
+                        _norm_filename(self._builtins_libspec_dir),
+                        prefix="gen_builtins_",
+                    ),
+                    timeout=100,
+                ):
+                    log.debug("Obtained mutex to generate builtins.")
+                    for libname in robot_constants.STDLIBS:
+                        builtins_libspec_dir = self._builtins_libspec_dir
+                        if not os.path.exists(
+                            os.path.join(builtins_libspec_dir, f"{libname}.libspec")
+                        ):
+                            wait_for.append(
+                                thread_pool.submit(
+                                    self._create_libspec, libname, is_builtin=True
+                                )
+                            )
+                    for future in wait_for:
+                        future.result()
+
+                if wait_for:
+                    log.debug(
+                        "Total time to generate builtins: %.2fs"
+                        % (time.time() - initial_time)
+                    )
+                    self.synchronize_internal_libspec_folders()
+            finally:
+                thread_pool.shutdown(wait=False)
         except:
             log.exception("Error creating builtin libraries.")
+        finally:
+            log.info("Finished creating builtin libraries.")
 
     def synchronize_workspace_folders(self):
         for folder_info in self._workspace_folder_uri_to_folder_info.values():
@@ -679,9 +708,13 @@ class LibspecManager(object):
 
                 libspec_filename = os.path.join(libspec_dir, libname + ".libspec")
 
+                log.debug(f"Obtaining mutex to generate libpsec: {libspec_filename}.")
                 with timed_acquire_mutex(
                     _get_libspec_mutex_name(libspec_filename)
                 ):  # Could fail.
+                    log.debug(
+                        f"Obtained mutex to generate libpsec: {libspec_filename}."
+                    )
                     call.append(libspec_filename)
 
                     mtime = -1
@@ -785,7 +818,7 @@ class LibspecManager(object):
         """
         libname_lower = libname.lower()
         if libname_lower.endswith((".py", ".class", ".java")):
-            libname_lower = os.path.splitext(libname)[0]
+            libname_lower = os.path.splitext(libname_lower)[0]
 
         if "/" in libname_lower or "\\" in libname_lower:
             libname_lower = os.path.basename(libname_lower)
