@@ -3,6 +3,8 @@ from robocorp_ls_core.basic import overrides
 from robocorp_ls_core.cache import instance_cache
 from robotframework_ls.constants import NULL
 from robocorp_ls_core.robotframework_log import get_logger
+from robotframework_ls.impl.protocols import IRobotWorkspace, IRobotDocument
+from robocorp_ls_core.protocols import check_implements
 
 log = get_logger(__name__)
 
@@ -29,6 +31,9 @@ class RobotWorkspace(Workspace):
     def _create_document(self, doc_uri, source=None, version=None):
         return RobotDocument(doc_uri, source, version, generate_ast=self._generate_ast)
 
+    def __typecheckself__(self) -> None:
+        _: IRobotWorkspace = check_implements(self)
+
 
 class RobotDocument(Document):
 
@@ -38,13 +43,14 @@ class RobotDocument(Document):
 
     def __init__(self, uri, source=None, version=None, generate_ast=True):
         Document.__init__(self, uri, source=source, version=version)
+
         self._generate_ast = generate_ast
         self._ast = None
 
     @overrides(Document._clear_caches)
     def _clear_caches(self):
         Document._clear_caches(self)
-        self.get_ast.cache_clear(self)
+        self.get_ast.cache_clear(self)  # noqa (clear the instance_cache).
 
     def get_type(self):
         path = self.path
@@ -69,7 +75,7 @@ class RobotDocument(Document):
             raise AssertionError(
                 "The AST can only be accessed in the RobotFrameworkServerApi, not in the RobotFrameworkLanguageServer."
             )
-        from robot.api import get_model, get_resource_model, get_init_model
+        from robot.api import get_model, get_resource_model, get_init_model  # noqa
 
         try:
             source = self.source
@@ -104,3 +110,6 @@ class RobotDocument(Document):
                 return i
         else:
             raise AssertionError(f"Did not find >>{contents}<< in doc.")
+
+    def __typecheckself__(self) -> None:
+        _: IRobotDocument = check_implements(self)
