@@ -1,6 +1,17 @@
 import sys
 import threading
-from typing import Dict, Union, Any, Generic, Callable, Mapping, Optional, List, Type
+from typing import (
+    Dict,
+    Union,
+    Any,
+    Generic,
+    Callable,
+    Mapping,
+    Optional,
+    List,
+    Type,
+    Iterator,
+)
 from typing import TypeVar
 import typing
 
@@ -112,11 +123,23 @@ class IMessageMatcher(Generic[T], Protocol):
     msg: T
 
 
+class IIdMessageMatcher(IMessageMatcher, Protocol):
+
+    message_id: str
+
+
 COMMUNICATION_DROPPED = CommunicationDropped()
 
 
-class ILanguageServerClientBase(Protocol):
-    def request_async(self, contents: Dict) -> IMessageMatcher:
+class IRequestCancellable(Protocol):
+    def request_cancel(self, message_id) -> None:
+        """
+        Requests that some processing is cancelled.
+        """
+
+
+class ILanguageServerClientBase(IRequestCancellable, Protocol):
+    def request_async(self, contents: Dict) -> Optional[IIdMessageMatcher]:
         """
         API which allows to wait for the message to complete.
         
@@ -178,6 +201,74 @@ class ILanguageServerClientBase(Protocol):
         pass
 
 
+class IRobotFrameworkApiClient(ILanguageServerClientBase, Protocol):
+    def initialize(
+        self, msg_id=None, process_id=None, root_uri="", workspace_folders=()
+    ):
+        pass
+
+    def get_version(self):
+        pass
+
+    def lint(self, doc_uri: str) -> list:
+        pass
+
+    def request_lint(self, doc_uri: str) -> Optional[IIdMessageMatcher]:
+        pass
+
+    def forward(self, method_name, params):
+        pass
+
+    def forward_async(self, method_name, params) -> Optional[IIdMessageMatcher]:
+        """
+        :Note: async complete.
+        """
+
+    def open(self, uri, version, source):
+        pass
+
+    def request_section_name_complete(
+        self, doc_uri, line, col
+    ) -> Optional[IIdMessageMatcher]:
+        """
+        :Note: async complete.
+        """
+
+    def request_keyword_complete(
+        self, doc_uri, line, col
+    ) -> Optional[IIdMessageMatcher]:
+        """
+        :Note: async complete.
+        """
+
+    def request_complete_all(self, doc_uri, line, col) -> Optional[IIdMessageMatcher]:
+        """
+        Completes: sectionName, keyword, variables
+        :Note: async complete.
+        """
+
+    def request_find_definition(
+        self, doc_uri, line, col
+    ) -> Optional[IIdMessageMatcher]:
+        """
+        :Note: async complete.
+        """
+
+    def request_source_format(
+        self, text_document, options
+    ) -> Optional[IIdMessageMatcher]:
+        """
+        :Note: async complete.
+        """
+
+    def request_signature_help(
+        self, doc_uri: str, line: int, col: int
+    ) -> Optional[IIdMessageMatcher]:
+        """
+        :Note: async complete.
+        """
+
+
 class ILanguageServerClient(ILanguageServerClientBase, Protocol):
     pid: Optional[int]
 
@@ -236,7 +327,12 @@ class ILanguageServerClient(ILanguageServerClientBase, Protocol):
     def execute_command(self, command: str, arguments: list) -> Mapping[str, Any]:
         pass
 
-    def execute_command_async(self, command: str, arguments: list) -> IMessageMatcher:
+    def execute_command_async(
+        self, command: str, arguments: list
+    ) -> Optional[IIdMessageMatcher]:
+        pass
+
+    def request_signature_help(self, uri: str, line: int, col: int):
         pass
 
 
@@ -335,18 +431,88 @@ class IDirCache(Protocol):
         """
 
 
+class IDocumentSelection(Protocol):
+
+    doc: "IDocument"
+    line: int
+    col: int
+
+    @property
+    def offset_at_position(self):
+        """Return the byte-offset pointed at by the given position."""
+        pass
+
+    @property
+    def current_line(self) -> str:
+        pass
+
+    @property
+    def line_to_column(self):
+        pass
+
+    @property
+    def word_at_column(self) -> str:
+        pass
+
+    @property
+    def word_to_column(self) -> str:
+        pass
+
+    @property
+    def word_from_column(self) -> str:
+        pass
+
+
 class IDocument(Protocol):
+    uri: str
+    version: Optional[str]
+    path: str
+
+    def selection(self, line: int, col: int) -> IDocumentSelection:
+        pass
+
+    @property
+    def source(self) -> str:
+        pass
+
+    @source.setter
+    def source(self, source: str) -> None:
+        pass
+
     def get_line(self, line: int) -> str:
         pass
 
-    def sync_source(self):
+    def is_source_in_sync(self) -> bool:
         """
-        If the document is backed up by a file, makes sure that it's contents
-        are synchronized with the filesystem.
+        If the document is backed up by a file, returns true if the sources are
+        in sync.
         """
 
 
+class IWorkspaceFolder(Protocol):
+    uri: str
+    name: str
+
+
 class IWorkspace(Protocol):
+    @property
+    def root_path(self):
+        pass
+
+    @property
+    def root_uri(self):
+        pass
+
+    def iter_documents(self) -> Iterator[IDocument]:
+        """
+        Note: the lock must be obtained when iterating documents.
+        """
+
+    def iter_folders(self) -> Iterator[IWorkspaceFolder]:
+        """
+        Note: the lock must be obtained when iterating folders.
+        """
+
     def remove_document(self, uri: str) -> None:
         pass
 
@@ -371,3 +537,13 @@ class IWorkspace(Protocol):
 class ITimeoutHandle(Protocol):
     def exec_on_timeout(self):
         pass
+
+
+class IMonitor(Protocol):
+    def cancel(self) -> None:
+        pass
+
+    def check_cancelled(self) -> None:
+        """
+        raises JsonRpcRequestCancelled if cancelled.
+        """
